//Libraries for Handling Audio Input Devices 
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <linux/soundcard.h>
//GTK Library
#include <gtk/gtk.h>
//General Purpose Libraries
#include <iostream>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
//Library for Handling Sound Files
#include <sndfile.h>
//using namespace std;
#include "omp.h"

#define	BUFFER_LEN	1024
#define MAX_CHANNELS	6

#define	PI	3.14159265358979323846264338
#define M_LN2 0.69314718055994530942
#define FILTER_SECTIONS   2  /* 2 filter sections for 24 db/oct filter */

/* whatever sample type you want */
typedef double smp_type;
/* this holds the data required to update samples thru a filter */
typedef struct {
smp_type a0, a1, a2, a3, a4;
smp_type x1, x2, y1, y2;
}
biquad;

void mainmenu();
int playback(int opt);
int record_sound(float length);
int plot_sound(int opt);
int make_command_plot(int opt);
static int tone_create (double sr, double interval_sec, double amplitude, double frequency, int channels);
static int amplify_data (double gain);
static int reverce_data ();
extern smp_type BiQuad(smp_type sample, biquad * b);

extern biquad *BiQuad_new(int type, smp_type dbGain, /* gain of filter */
smp_type freq, /* center frequency */
smp_type srate, /* sampling rate */
smp_type q); /* quality factor */

int bandpass_filter();
int stopband_filter();
static int filter();


/* filter types */
enum {
LPF, /* low pass filter */
HPF, /* High pass filter */
BPF, /* band pass filter */
NOTCH, /* Notch Filter */
PEQ, /* Peaking band EQ filter */
LSH, /* Low shelf filter */
HSH /* High shelf filter */
};



static double data [BUFFER_LEN];
SNDFILE *infile, *outfile;
SF_INFO sfinfo ;
SF_FORMAT_INFO info;

biquad *flt;
biquad *secondary_flt;

double   fs, fc;     /* Sampling frequency, cutoff frequency OR Sampling frequency, frequency of Tone*/
double   fl,fu;		/* Used for BandPass and StopBand filters */
double   k;           /* overall gain factor OR amplitude of Tone OR The amplification factor*/
double   Q;     /* Resonance > 1.0 < 1000 OR interval of Tone*/
double   timeplot; /*The time for ploting the samples*/
double   work;		/*GENERAL PURPOSE VARIABLE */

int	readcount, err,opt1,opt2,chan,stopmenu,stopplay;

char *infilename=NULL;
char *outfilename=NULL;
char command[2000];
char message[3000];

/*GTK Objects Global*/
void btn_play_input();
void btn_stop_sound();
void btn_play_output();
void btn_plot_input();
void btn_plot_output();
void btn_rec();
void btn_rec_stop();
void btn_LPF();
void btn_HPF();
void btn_BPF();
void btn_BSF();
void btn_NOTCH();
void btn_LSF();
void btn_HSF();
void btn_reverce();
void btn_filter1();
void btn_filter2();
void btn_amplify();
void btn_tone();
void btn_TestSetThreads();

GtkWidget *window;
GtkWidget *fixed_window; /*The fixed control of the main window*/
GtkWidget *reverce_btn;

GtkWidget *input_frame;/*The input frame */
GtkWidget *fixed_input_frame; /*The fixed control of the input frame */
GtkWidget *InputFilelabel;
GtkWidget *input_entry;
GtkWidget *input_play;
GtkWidget *plot_input;/*Plot input button*/

GtkWidget *output_frame;/*The output frame */
GtkWidget *fixed_output_frame; /*The fixed control of the output frame */
GtkWidget *OutputFilelabel;
GtkWidget *output_entry;
GtkWidget *output_play;
GtkWidget *plot_output;/*Plot output button*/

GtkWidget *rec_frame;/*The record frame */
GtkWidget *fixed_rec_frame; /*The fixed control of the record frame */
GtkWidget *duration_entry;
GtkWidget *DurationLabel;
GtkWidget *rec_start;
GtkWidget *rec_stop;

GtkWidget *filter_frame;/*The filter frame */
GtkWidget *fixed_filter_frame; /*The fixed control of the filter frame */
GtkWidget *lowpass_btn;/*lowpass button*/
GtkWidget *highpass_btn;/*highpass button*/
GtkWidget *bandpass_btn;/*bandpass button*/
GtkWidget *bandstop_btn;/*bandstop button*/
GtkWidget *notch_btn;/*notch button*/
GtkWidget *lowshelf_btn;/*notch button*/
GtkWidget *highself_btn;/*notch button*/

GtkWidget *amplify_frame;/*The amplify frame */
GtkWidget *fixed_amplify_frame; /*The fixed control of the amplify frame */
GtkWidget *amplify_btn;/*amplify button*/
GtkWidget *AmplifyLabel;/*amplify Label*/
GtkWidget *amplify_entry;/*amplify entry*/

GtkWidget *tone_frame;/*The tone frame */
GtkWidget *fixed_tone_frame; /*The fixed control of the tone frame */
GtkWidget *tone_btn;/*amplify button*/
GtkWidget *ToneFrequencyLabel;/*tone frequency Label*/
GtkWidget *ToneDurationLabel;/*tone duration Label*/
GtkWidget *ToneAmplitudeLabel;/*tone amplitude Label*/
GtkWidget *ToneFrequency_entry;/*tone frequency entry*/
GtkWidget *ToneDuration_entry;/*tone duration entry*/
GtkWidget *ToneAmplitude_entry;/*tone amplitude entry*/

GtkWidget *view;/*View control*/
GtkTextBuffer *buffer;
GtkWidget *scrolled_window;

/*Threads parameters*/
GtkWidget *InputNumThread_entry;
GtkWidget *NumThreadLabel;
GtkWidget *TestSetThreads_btn;

/*Stop sound window*/
GtkWidget *window2;
GtkWidget *label2;
GtkWidget *fixed_window2; /*The fixed control of the stop window*/
GtkWidget *button2;
/*--END-- Stop sound window*/

/*Filter window*/
GtkWidget *window3;
GtkWidget *fixed_window3; /*The fixed control of the filter window*/
GtkWidget *fs_label;
GtkWidget *fs_entry;
GtkWidget *fc_label;
GtkWidget *fc_entry;
GtkWidget *fu_label;
GtkWidget *fu_entry;
GtkWidget *fl_label;
GtkWidget *fl_entry;
GtkWidget *submit_btn;
/*--END-- Filter Window*/

/*Image objects*/
GtkWidget *pixmapplayin;
GtkWidget *pixmapplayout;
GtkWidget *pixmaprec;
GtkWidget *pixmapplaystop;
GtkWidget *pixmaprecstop;

static GtkTextIter startiter;
GtkTextIter stopiter;
gint offset;

static void destroy_window( GtkWidget *widget,gpointer  pointer ){
     gtk_main_quit ();
}

static void print_message(char* message){
gtk_text_buffer_get_iter_at_offset (buffer, &startiter, 0);
gtk_text_buffer_get_iter_at_offset (buffer, &stopiter, -1);
gtk_text_buffer_insert  (buffer, &startiter, message, -1);
}

static void close_window2(){
gtk_widget_destroy(window2);
}

static void close_window3(){
gtk_widget_destroy(window3);
}

static void stop_window(){
	window2 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
gtk_window_set_title(GTK_WINDOW(window2), "Stop sound");
gtk_window_set_default_size (GTK_WINDOW (window2), 250, 140);
g_signal_connect (G_OBJECT (window2), "destroy",G_CALLBACK (destroy_window), NULL);
gtk_container_set_border_width (GTK_CONTAINER (window2), 10);

fixed_window2=gtk_fixed_new();
gtk_container_add (GTK_CONTAINER (window2), fixed_window2);
gtk_widget_show(fixed_window2); 

label2 = gtk_label_new ("Press the stop button to stop sound\nor recording");
gtk_label_set_justify(GTK_LABEL (label2),GTK_JUSTIFY_CENTER );
gtk_fixed_put (GTK_FIXED (fixed_window2),label2 , 0, 0);
gtk_widget_show(label2);

pixmapplaystop = gtk_image_new_from_file("stop.jpg");

button2 = gtk_button_new();
gtk_signal_connect (GTK_OBJECT (button2), "clicked", GTK_SIGNAL_FUNC (btn_stop_sound), fixed_window2);

gtk_container_add( GTK_CONTAINER(button2), pixmapplaystop );
gtk_fixed_put (GTK_FIXED (fixed_window2), button2, 85, 35);
gtk_widget_show( pixmapplaystop );
gtk_widget_show( button2 );	
gtk_window_set_resizable (GTK_WINDOW(window2), false);
gtk_window_set_deletable  (GTK_WINDOW(window2), false);

gtk_widget_show_all (window2);
gtk_main ();
}

static void filter_window(int type, char *title){
window3 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
gtk_window_set_title(GTK_WINDOW(window3), title);
gtk_window_set_default_size (GTK_WINDOW (window3), 260, 310);
g_signal_connect (G_OBJECT (window3), "destroy",G_CALLBACK (destroy_window), NULL);
gtk_container_set_border_width (GTK_CONTAINER (window3), 10);

fixed_window3=gtk_fixed_new();
gtk_container_add (GTK_CONTAINER (window3), fixed_window3);
gtk_widget_show(fixed_window3); 

fs_label = gtk_label_new ("Sample frequency (in HZ):");
gtk_label_set_justify(GTK_LABEL (fs_label),GTK_JUSTIFY_RIGHT );
gtk_fixed_put (GTK_FIXED (fixed_window3),fs_label , 0, 7);
gtk_widget_show(fs_label);

fs_entry= gtk_entry_new();
gtk_entry_set_max_length(GTK_ENTRY(fs_entry), 15);
gtk_entry_set_width_chars(GTK_ENTRY(fs_entry), 7);
gtk_entry_set_text(GTK_ENTRY(fs_entry),"44100");
gtk_fixed_put (GTK_FIXED (fixed_window3), fs_entry, 175, 2); 
gtk_widget_show( fs_entry );

if (type==1){
fc_label = gtk_label_new ("Cutoff frequency (fc in HZ):");
gtk_label_set_justify(GTK_LABEL (fc_label),GTK_JUSTIFY_RIGHT );
gtk_fixed_put (GTK_FIXED (fixed_window3),fc_label , 0, 47);
gtk_widget_show(fc_label);

fc_entry= gtk_entry_new();
gtk_entry_set_max_length(GTK_ENTRY(fc_entry), 15);
gtk_entry_set_width_chars(GTK_ENTRY(fc_entry), 7);
gtk_entry_set_text(GTK_ENTRY(fc_entry),"");
gtk_fixed_put (GTK_FIXED (fixed_window3), fc_entry, 175, 40); 
gtk_widget_show( fc_entry );
}else if (type==2){

fl_label = gtk_label_new ("Lower frequency (fl in HZ):");
gtk_label_set_justify(GTK_LABEL (fl_label),GTK_JUSTIFY_RIGHT );
gtk_fixed_put (GTK_FIXED (fixed_window3),fl_label , 0, 87);
gtk_widget_show(fl_label);

fl_entry= gtk_entry_new();
gtk_entry_set_max_length(GTK_ENTRY(fl_entry), 15);
gtk_entry_set_width_chars(GTK_ENTRY(fl_entry), 7);
gtk_entry_set_text(GTK_ENTRY(fl_entry),"");
gtk_fixed_put (GTK_FIXED (fixed_window3), fl_entry, 175, 80); 
gtk_widget_show( fl_entry );

fu_label = gtk_label_new ("Upper frequency (fu in HZ):");
gtk_label_set_justify(GTK_LABEL (fu_label),GTK_JUSTIFY_RIGHT );
gtk_fixed_put (GTK_FIXED (fixed_window3),fu_label , 0, 127);
gtk_widget_show(fu_label);

fu_entry= gtk_entry_new();
gtk_entry_set_max_length(GTK_ENTRY(fu_entry), 15);
gtk_entry_set_width_chars(GTK_ENTRY(fu_entry), 7);
gtk_entry_set_text(GTK_ENTRY(fu_entry),"");
gtk_fixed_put (GTK_FIXED (fixed_window3), fu_entry, 175, 120); 
gtk_widget_show( fu_entry );
}

submit_btn = gtk_button_new_with_label ("  Filter  ");
if (type==1){
gtk_signal_connect (GTK_OBJECT (submit_btn), "clicked", GTK_SIGNAL_FUNC (btn_filter1), fixed_window3);
}else if (type==2){
gtk_signal_connect (GTK_OBJECT (submit_btn), "clicked", GTK_SIGNAL_FUNC (btn_filter2), fixed_window3);
}
gtk_fixed_put (GTK_FIXED (fixed_window3), submit_btn, 0, 155);	

gtk_window_set_resizable (GTK_WINDOW(window3), false);
gtk_window_set_deletable  (GTK_WINDOW(window3), true);

gtk_widget_show_all (window3);
gtk_main ();	
}
/*--END-- GTK objects*/

/*OpneMP Variables*/
long n=omp_get_num_procs();
int nthreads, tid, current_tid;
omp_lock_t lock;
double timepass;
/*--END--OpneMP Variables*/

int main(int argc, char *argv[])
{

/*Initialize variables */	
err=1;
current_tid=0;
stopplay=1;
opt1=0;
opt2=0;
nthreads=2;
omp_set_num_threads(nthreads);
/*--END-- Initialize variables */

gtk_init (&argc, &argv);
window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
gtk_window_set_title(GTK_WINDOW(window), "Sound Process");
gtk_window_set_default_size (GTK_WINDOW (window), 750, 600);
g_signal_connect (G_OBJECT (window), "destroy",G_CALLBACK (destroy_window), NULL);
gtk_container_set_border_width (GTK_CONTAINER (window), 10);

fixed_window=gtk_fixed_new();
gtk_container_add (GTK_CONTAINER (window), fixed_window);
gtk_widget_show(fixed_window); 

/*Initialize image objects */
pixmapplayin = gtk_image_new_from_file("play.jpg");
pixmapplayout = gtk_image_new_from_file("play.jpg");
pixmaprec = gtk_image_new_from_file("rec.jpg");
/*--END-- Initialize image objects */

/*Create input frame and place objects in it */
input_frame = gtk_frame_new(NULL);
gtk_frame_set_label( GTK_FRAME(input_frame), "Input File" );
gtk_frame_set_label_align( GTK_FRAME(input_frame), 0.0, 0.0);
gtk_fixed_put (GTK_FIXED (fixed_window),input_frame , 10, 10);
gtk_frame_set_shadow_type( GTK_FRAME(input_frame), GTK_SHADOW_IN);
gtk_widget_set_usize(input_frame,720,80);
gtk_widget_show(input_frame);

fixed_input_frame=gtk_fixed_new();
gtk_container_add (GTK_CONTAINER (input_frame), fixed_input_frame);
gtk_widget_show(fixed_input_frame); 

InputFilelabel = gtk_label_new ("Input file path:");
gtk_label_set_justify(GTK_LABEL (InputFilelabel),GTK_JUSTIFY_RIGHT );
gtk_fixed_put (GTK_FIXED (fixed_input_frame),InputFilelabel , 10, 15);
gtk_widget_show(InputFilelabel);

input_entry= gtk_entry_new();
gtk_entry_set_max_length(GTK_ENTRY(input_entry), 250);
gtk_entry_set_width_chars(GTK_ENTRY(input_entry), 40);
gtk_entry_set_text(GTK_ENTRY(input_entry),"");
gtk_fixed_put (GTK_FIXED (fixed_input_frame), input_entry, 115, 12); 
gtk_widget_show( input_entry );

input_play = gtk_button_new();
gtk_signal_connect (GTK_OBJECT (input_play), "clicked", GTK_SIGNAL_FUNC (btn_play_input), fixed_input_frame);
gtk_container_add( GTK_CONTAINER(input_play), pixmapplayin );
gtk_fixed_put (GTK_FIXED (fixed_input_frame), input_play, 460, 4);
gtk_widget_show( pixmapplayin );
gtk_widget_show( input_play );

plot_input = gtk_button_new_with_label ("Plot input wave");
gtk_signal_connect (GTK_OBJECT (plot_input), "clicked", GTK_SIGNAL_FUNC (btn_plot_input), fixed_input_frame);
gtk_fixed_put (GTK_FIXED (fixed_input_frame), plot_input, 525, 12);	
/*--END-- Create input frame and place objects in it */

/*Create output frame and place objects in it */
output_frame = gtk_frame_new(NULL);
gtk_widget_set_usize(output_frame,720,80);
gtk_frame_set_label( GTK_FRAME(output_frame), "Output File" );
gtk_frame_set_label_align( GTK_FRAME(output_frame), 0.0, 0.0);
gtk_fixed_put (GTK_FIXED (fixed_window),output_frame , 10, 100);
gtk_frame_set_shadow_type( GTK_FRAME(output_frame), GTK_SHADOW_IN);
gtk_widget_show(output_frame);

fixed_output_frame=gtk_fixed_new();
gtk_container_add (GTK_CONTAINER (output_frame), fixed_output_frame);
gtk_widget_show(fixed_output_frame); 

OutputFilelabel = gtk_label_new ("Output file path:");
gtk_label_set_justify(GTK_LABEL (OutputFilelabel),GTK_JUSTIFY_RIGHT );
gtk_fixed_put (GTK_FIXED (fixed_output_frame),OutputFilelabel , 10, 15);
gtk_widget_show(OutputFilelabel);

output_entry= gtk_entry_new();
gtk_entry_set_max_length(GTK_ENTRY(output_entry), 250);
gtk_entry_set_width_chars(GTK_ENTRY(output_entry), 40);
gtk_entry_set_text(GTK_ENTRY(output_entry),"");
gtk_fixed_put (GTK_FIXED (fixed_output_frame), output_entry, 115, 12); 
gtk_widget_show( output_entry );

output_play = gtk_button_new();
gtk_signal_connect (GTK_OBJECT (output_play), "clicked", GTK_SIGNAL_FUNC (btn_play_output), fixed_output_frame);
gtk_container_add( GTK_CONTAINER(output_play), pixmapplayout );
gtk_fixed_put (GTK_FIXED (fixed_output_frame), output_play, 460, 4);
gtk_widget_show( pixmapplayout );
gtk_widget_show( output_play );	

plot_output = gtk_button_new_with_label ("Plot output wave");
gtk_signal_connect (GTK_OBJECT (plot_output), "clicked", GTK_SIGNAL_FUNC (btn_plot_output), fixed_output_frame);
gtk_fixed_put (GTK_FIXED (fixed_output_frame), plot_output, 525, 12);		
/*--END-- Create output frame and place objects in it */

/*Create rec frame and place objects in it */
rec_frame = gtk_frame_new(NULL);
gtk_widget_set_usize(rec_frame,240,110);
gtk_frame_set_label( GTK_FRAME(rec_frame), "Record from mic" );
gtk_frame_set_label_align( GTK_FRAME(rec_frame), 0.0, 0.0);
gtk_fixed_put (GTK_FIXED (fixed_window),rec_frame , 10, 190);
gtk_frame_set_shadow_type( GTK_FRAME(rec_frame), GTK_SHADOW_IN);
gtk_widget_show(rec_frame);

fixed_rec_frame=gtk_fixed_new();
gtk_container_add (GTK_CONTAINER (rec_frame), fixed_rec_frame);
gtk_widget_show(fixed_rec_frame); 

DurationLabel = gtk_label_new ("Recording duration (in sec):");
gtk_label_set_justify(GTK_LABEL (DurationLabel),GTK_JUSTIFY_RIGHT );
gtk_fixed_put (GTK_FIXED (fixed_rec_frame),DurationLabel , 0, 4);
gtk_widget_show(OutputFilelabel);

duration_entry= gtk_entry_new();
gtk_entry_set_max_length(GTK_ENTRY(duration_entry), 8);
gtk_entry_set_width_chars(GTK_ENTRY(duration_entry), 5);
gtk_entry_set_text(GTK_ENTRY(duration_entry),"1");
gtk_fixed_put (GTK_FIXED (fixed_rec_frame), duration_entry, 175, 0); 
gtk_widget_show( duration_entry );

rec_start = gtk_button_new();
gtk_signal_connect (GTK_OBJECT (rec_start), "clicked", GTK_SIGNAL_FUNC (btn_rec), fixed_rec_frame);
gtk_container_add( GTK_CONTAINER(rec_start), pixmaprec );
gtk_fixed_put (GTK_FIXED (fixed_rec_frame), rec_start, 80, 25);
gtk_widget_show( pixmaprec );
gtk_widget_show( rec_start );

/*
rec_stop = gtk_button_new();
pixmaprecstop = gtk_image_new_from_file("stop.jpg");
gtk_signal_connect (GTK_OBJECT (rec_stop), "clicked", GTK_SIGNAL_FUNC (btn_rec_stop), fixed_rec_frame);
gtk_container_add( GTK_CONTAINER(rec_stop), pixmaprecstop );
gtk_fixed_put (GTK_FIXED (fixed_rec_frame), rec_stop, 10, 25);
gtk_widget_show( pixmaprecstop );
gtk_widget_show( rec_stop );
*/
gtk_widget_show( rec_start );	
/*--END--Create rec frame and place objects in it */

/*Create filter frame*/
filter_frame = gtk_frame_new(NULL);
gtk_widget_set_usize(filter_frame,280,110);
gtk_frame_set_label( GTK_FRAME(filter_frame), "Digital filters" );
gtk_frame_set_label_align( GTK_FRAME(filter_frame), 0.0, 0.0);
gtk_fixed_put (GTK_FIXED (fixed_window),filter_frame , 260, 190);
gtk_frame_set_shadow_type( GTK_FRAME(filter_frame), GTK_SHADOW_IN);
gtk_widget_show(filter_frame);

fixed_filter_frame=gtk_fixed_new();
gtk_container_add (GTK_CONTAINER (filter_frame), fixed_filter_frame);
gtk_widget_show(fixed_filter_frame); 

lowpass_btn = gtk_button_new_with_label ("  LPF  ");
gtk_signal_connect (GTK_OBJECT (lowpass_btn), "clicked", GTK_SIGNAL_FUNC (btn_LPF), fixed_filter_frame);
gtk_fixed_put (GTK_FIXED (fixed_filter_frame), lowpass_btn, 10, 8);

highpass_btn = gtk_button_new_with_label ("  HPF  ");
gtk_signal_connect (GTK_OBJECT (highpass_btn), "clicked", GTK_SIGNAL_FUNC (btn_HPF), fixed_filter_frame);
gtk_fixed_put (GTK_FIXED (fixed_filter_frame), highpass_btn, 70, 8);

bandpass_btn = gtk_button_new_with_label ("  BPF  ");
gtk_signal_connect (GTK_OBJECT (bandpass_btn), "clicked", GTK_SIGNAL_FUNC (btn_BPF), fixed_filter_frame);
gtk_fixed_put (GTK_FIXED (fixed_filter_frame), bandpass_btn, 140, 8);

bandstop_btn = gtk_button_new_with_label ("  BSF  ");
gtk_signal_connect (GTK_OBJECT (bandstop_btn), "clicked", GTK_SIGNAL_FUNC (btn_BSF), fixed_filter_frame);
gtk_fixed_put (GTK_FIXED (fixed_filter_frame), bandstop_btn, 210, 8);

notch_btn = gtk_button_new_with_label (" NOTCH ");
gtk_signal_connect (GTK_OBJECT (notch_btn), "clicked", GTK_SIGNAL_FUNC (btn_NOTCH), fixed_filter_frame);
gtk_fixed_put (GTK_FIXED (fixed_filter_frame), notch_btn, 10, 50);	

lowshelf_btn = gtk_button_new_with_label ("  LSF  ");
gtk_signal_connect (GTK_OBJECT (lowshelf_btn), "clicked", GTK_SIGNAL_FUNC (btn_LSF), fixed_filter_frame);
gtk_fixed_put (GTK_FIXED (fixed_filter_frame), lowshelf_btn, 110, 50);	

highself_btn = gtk_button_new_with_label ("  HSF  ");
gtk_signal_connect (GTK_OBJECT (highself_btn), "clicked", GTK_SIGNAL_FUNC (btn_HSF), fixed_filter_frame);
gtk_fixed_put (GTK_FIXED (fixed_filter_frame), highself_btn, 180, 50);	
/*--END-- Create filter frame*/

/*Create amplify frame*/
amplify_frame = gtk_frame_new(NULL);
gtk_widget_set_usize(amplify_frame,200,95);
gtk_frame_set_label( GTK_FRAME(amplify_frame), "Amplify input" );
gtk_frame_set_label_align( GTK_FRAME(amplify_frame), 0.0, 0.0);
gtk_fixed_put (GTK_FIXED (fixed_window),amplify_frame , 10, 310);
gtk_frame_set_shadow_type( GTK_FRAME(amplify_frame), GTK_SHADOW_IN);
gtk_widget_show(amplify_frame);

fixed_amplify_frame=gtk_fixed_new();
gtk_container_add (GTK_CONTAINER (amplify_frame), fixed_amplify_frame);
gtk_widget_show(fixed_amplify_frame); 

AmplifyLabel = gtk_label_new ("Amplification factor:");
gtk_label_set_justify(GTK_LABEL (AmplifyLabel),GTK_JUSTIFY_RIGHT );
gtk_fixed_put (GTK_FIXED (fixed_amplify_frame),AmplifyLabel , 10, 15);
gtk_widget_show(AmplifyLabel);

amplify_entry= gtk_entry_new();
gtk_entry_set_max_length(GTK_ENTRY(amplify_entry), 5);
gtk_entry_set_width_chars(GTK_ENTRY(amplify_entry), 4);
gtk_entry_set_text(GTK_ENTRY(amplify_entry),"1");
gtk_fixed_put (GTK_FIXED (fixed_amplify_frame), amplify_entry, 140, 10); 
gtk_widget_show( amplify_entry );

amplify_btn = gtk_button_new_with_label ("Amplify");
gtk_signal_connect (GTK_OBJECT (amplify_btn), "clicked", GTK_SIGNAL_FUNC (btn_amplify), fixed_amplify_frame);
gtk_fixed_put (GTK_FIXED (fixed_amplify_frame), amplify_btn, 10, 45);	
/*--END-- Create amplify frame*/

/*Create amplify frame*/
tone_frame = gtk_frame_new(NULL);
gtk_widget_set_usize(tone_frame,320,95);
gtk_frame_set_label( GTK_FRAME(tone_frame), "Create Tone" );
gtk_frame_set_label_align( GTK_FRAME(tone_frame), 0.0, 0.0);
gtk_fixed_put (GTK_FIXED (fixed_window),tone_frame , 220, 310);
gtk_frame_set_shadow_type( GTK_FRAME(tone_frame), GTK_SHADOW_IN);
gtk_widget_show(tone_frame);

fixed_tone_frame=gtk_fixed_new();
gtk_container_add (GTK_CONTAINER (tone_frame), fixed_tone_frame);
gtk_widget_show(fixed_tone_frame); 

ToneFrequencyLabel = gtk_label_new ("Tone frequency (Hz):");
gtk_label_set_justify(GTK_LABEL (ToneFrequencyLabel),GTK_JUSTIFY_RIGHT );
gtk_fixed_put (GTK_FIXED (fixed_tone_frame),ToneFrequencyLabel , 10, 5);
gtk_widget_show(ToneFrequencyLabel);

ToneDurationLabel = gtk_label_new ("Tone duration (sec):");
gtk_label_set_justify(GTK_LABEL (ToneDurationLabel),GTK_JUSTIFY_RIGHT );
gtk_fixed_put (GTK_FIXED (fixed_tone_frame),ToneDurationLabel , 10, 30);
gtk_widget_show(ToneDurationLabel);

ToneAmplitudeLabel = gtk_label_new ("Tone amplitude Vp-p (max=2):");
gtk_label_set_justify(GTK_LABEL (ToneAmplitudeLabel),GTK_JUSTIFY_RIGHT );
gtk_fixed_put (GTK_FIXED (fixed_tone_frame),ToneAmplitudeLabel , 10, 55);
gtk_widget_show(ToneAmplitudeLabel);

ToneFrequency_entry= gtk_entry_new();
gtk_entry_set_max_length(GTK_ENTRY(ToneFrequency_entry), 6);
gtk_entry_set_width_chars(GTK_ENTRY(ToneFrequency_entry), 5);
gtk_entry_set_text(GTK_ENTRY(ToneFrequency_entry),"");
gtk_fixed_put (GTK_FIXED (fixed_tone_frame), ToneFrequency_entry, 145, 0); 
gtk_widget_show( ToneFrequency_entry );

ToneDuration_entry= gtk_entry_new();
gtk_entry_set_max_length(GTK_ENTRY(ToneDuration_entry), 6);
gtk_entry_set_width_chars(GTK_ENTRY(ToneDuration_entry), 5);
gtk_entry_set_text(GTK_ENTRY(ToneDuration_entry),"");
gtk_fixed_put (GTK_FIXED (fixed_tone_frame), ToneDuration_entry, 145, 27); 
gtk_widget_show( ToneDuration_entry );

ToneAmplitude_entry= gtk_entry_new();
gtk_entry_set_max_length(GTK_ENTRY(ToneAmplitude_entry), 6);
gtk_entry_set_width_chars(GTK_ENTRY(ToneAmplitude_entry), 5);
gtk_entry_set_text(GTK_ENTRY(ToneAmplitude_entry),"");
gtk_fixed_put (GTK_FIXED (fixed_tone_frame), ToneAmplitude_entry, 210, 50); 
gtk_widget_show( ToneAmplitude_entry );

tone_btn = gtk_button_new_with_label ("Create tone");
gtk_signal_connect (GTK_OBJECT (tone_btn), "clicked", GTK_SIGNAL_FUNC (btn_tone), fixed_tone_frame);
gtk_fixed_put (GTK_FIXED (fixed_tone_frame), tone_btn, 210, 0);	
/*--END-- Create amplify frame*/

/*Start Thread Control Objects*/
InputNumThread_entry= gtk_entry_new();
gtk_entry_set_max_length(GTK_ENTRY(InputNumThread_entry), 6);
gtk_entry_set_width_chars(GTK_ENTRY(InputNumThread_entry), 5);
gtk_entry_set_text(GTK_ENTRY(InputNumThread_entry),"2");
gtk_fixed_put (GTK_FIXED (fixed_window), InputNumThread_entry, 550, 255); 
gtk_widget_show( InputNumThread_entry );

NumThreadLabel = gtk_label_new ("Set Number of Threads:");
gtk_label_set_justify(GTK_LABEL (NumThreadLabel),GTK_JUSTIFY_RIGHT );
gtk_fixed_put (GTK_FIXED (fixed_window),NumThreadLabel , 550, 235);
gtk_widget_show(NumThreadLabel);

TestSetThreads_btn=gtk_button_new_with_label ("Test/Set Threads");
gtk_signal_connect (GTK_OBJECT (TestSetThreads_btn), "clicked", GTK_SIGNAL_FUNC (btn_TestSetThreads), fixed_tone_frame);
gtk_fixed_put (GTK_FIXED (fixed_window), TestSetThreads_btn, 550, 285);	

/*end Thread Control Objects*/

view = gtk_text_view_new ();
buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
gtk_text_buffer_set_text (buffer, "", -1);
   
scrolled_window = gtk_scrolled_window_new (NULL, NULL);
gtk_container_set_border_width (GTK_CONTAINER (scrolled_window), 10);
gtk_widget_set_usize(scrolled_window, 700, 200);
 
gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrolled_window), view);
gtk_fixed_put (GTK_FIXED (fixed_window), scrolled_window, 10, 400);
gtk_window_set_resizable (GTK_WINDOW(window), true);

reverce_btn = gtk_button_new_with_label ("Reverce input wave");
gtk_signal_connect (GTK_OBJECT (reverce_btn), "clicked", GTK_SIGNAL_FUNC (btn_reverce), fixed_window);
gtk_fixed_put (GTK_FIXED (fixed_window), reverce_btn, 550, 190);	

gtk_widget_show_all (window);
gtk_main ();

//mainmenu();
	
return err;
}

void btn_play_input(){
infilename=(char*)(gtk_entry_get_text(GTK_ENTRY(input_entry)));
opt1=5;
opt2=1;
if (stopplay==1 && strlen(infilename)>3){
mainmenu();
}
return;
}

void btn_play_output(){
outfilename=(char*)(gtk_entry_get_text(GTK_ENTRY(output_entry)));
opt1=5;
opt2=2;
if (stopplay==1 && strlen(outfilename)>3){
mainmenu();
}
return;
}

void btn_stop_sound(){
stopplay=1;
system("kill -9 $(pidof play)");
system("kill -9 $(pidof rec)");
return;
}

void btn_plot_input(){
infilename=(char*)(gtk_entry_get_text(GTK_ENTRY(input_entry)));
opt1=6;
opt2=1;
mainmenu();
return;
}

void btn_plot_output(){
outfilename=(char*)(gtk_entry_get_text(GTK_ENTRY(output_entry)));
opt1=6;
opt2=2;
mainmenu();
return;
}

void btn_rec(){
outfilename=(char*)(gtk_entry_get_text(GTK_ENTRY(output_entry)));
work= atof(gtk_entry_get_text(GTK_ENTRY(duration_entry)));

opt1=7;
opt2=0;
if (stopplay==1 && strlen(outfilename)>3){
mainmenu();
}
return;	
}

void btn_rec_stop(){
system("killall rec");	
return;
}

void btn_LPF(){
opt1=4;
opt2=1;
char Window_Title[]="Low Pass Filter";
filter_window(1,Window_Title);	
return;
}

void btn_HPF(){
opt1=4;
opt2=2;
char Window_Title[]="High Pass Filter";
filter_window(1,Window_Title);	
return;
}

void btn_BPF(){
opt1=4;
opt2=3;
char Window_Title[]="Band Pass Filter";
filter_window(2,Window_Title);	
return;
}

void btn_BSF(){
opt1=4;
opt2=4;
char Window_Title[]="Band Stop Filter";
filter_window(2,Window_Title);	
return;
}

void btn_NOTCH(){
opt1=4;
opt2=5;
char Window_Title[]="Notch Filter";
filter_window(1,Window_Title);
return;
}

void btn_LSF(){
opt1=4;
opt2=6;
char Window_Title[]="Low Shelf Filter";
filter_window(1,Window_Title);
return;
}

void btn_HSF(){
opt1=4;
opt2=7;
char Window_Title[]="High Self Filter";
filter_window(1,Window_Title);
return;
}

void btn_amplify(){
infilename=(char*)(gtk_entry_get_text(GTK_ENTRY(input_entry)));
outfilename=(char*)(gtk_entry_get_text(GTK_ENTRY(output_entry)));
k=atof(gtk_entry_get_text(GTK_ENTRY(amplify_entry)));
opt1=2;
opt2=0;
mainmenu();
return;
}

void btn_tone(){
outfilename=(char*)(gtk_entry_get_text(GTK_ENTRY(output_entry)));
fc=atof(gtk_entry_get_text(GTK_ENTRY(ToneFrequency_entry)));
Q=atof(gtk_entry_get_text(GTK_ENTRY(ToneDuration_entry)));
k=atof(gtk_entry_get_text(GTK_ENTRY(ToneAmplitude_entry)));
opt1=3;
opt2=0;
mainmenu();
return;
}

void btn_filter1(){
infilename=(char*)(gtk_entry_get_text(GTK_ENTRY(input_entry)));	
outfilename=(char*)(gtk_entry_get_text(GTK_ENTRY(output_entry)));
fs=atof(gtk_entry_get_text(GTK_ENTRY(fs_entry)));
fc=atof(gtk_entry_get_text(GTK_ENTRY(fc_entry)));
if (fc<=16){
fc=16;
}
mainmenu();

return;
}
void btn_filter2(){
infilename=(char*)(gtk_entry_get_text(GTK_ENTRY(input_entry)));	
outfilename=(char*)(gtk_entry_get_text(GTK_ENTRY(output_entry)));
fs=atof(gtk_entry_get_text(GTK_ENTRY(fs_entry)));
fl=atof(gtk_entry_get_text(GTK_ENTRY(fl_entry)));
fu=atof(gtk_entry_get_text(GTK_ENTRY(fu_entry)));
if (fl<=16){
fl=16;
}
mainmenu();
return;
}

void btn_reverce(){
infilename=(char*)(gtk_entry_get_text(GTK_ENTRY(input_entry)));	
outfilename=(char*)(gtk_entry_get_text(GTK_ENTRY(output_entry)));
opt1=1;
opt2=0;
mainmenu();
return;
}

void btn_TestSetThreads(){
strcpy(message,"");
nthreads=atoi(gtk_entry_get_text(GTK_ENTRY(InputNumThread_entry)));
if (nthreads<=2){
nthreads=2;
strcpy(&message[strlen(message)],"Number of threads set to 2. The minimum allowed thread number\n\n");
}else{
strcpy(&message[strlen(message)],"Number of threads set successfully to ");	
strcpy(&message[strlen(message)],gtk_entry_get_text(GTK_ENTRY(InputNumThread_entry)));	
strcpy(&message[strlen(message)],"\n\n");	
strcpy(&message[strlen(message)],"Testing threads...\n ");	
}

omp_set_num_threads(nthreads);
omp_init_lock(&lock);
#pragma omp parallel private(tid,current_tid) shared(nthreads,message,lock)   
{
	omp_set_lock(&lock);

  tid = omp_get_thread_num();
  if (tid == 0) 
    {    
    sprintf(&message[strlen(message)],"Running thread = %d\n", tid);
    sprintf(&message[strlen(message)],"      Number of threads = %d\n", nthreads);
    }else{
	sprintf(&message[strlen(message)],"Running thread = %d\n", tid);
	}
	
	omp_unset_lock(&lock);
}
omp_destroy_lock(&lock);

print_message(message);
char tmp[]="\n";
print_message(tmp);
}

void mainmenu(){
strcpy(message,"");	
	//stopmenu=0;
    //opt1=0;
	//opt2=0;
	
		//printf("Enter the name and the path of the sound file to be processed, input file *.wav .\n:>>");
		//scanf("%s",infilename);
		//printf("Enter the name and the path of the output file *.wav .\n:>>");
		//scanf("%s",outfilename);
	//while(!stopmenu){
		//printf("\n");
		//printf("Select 1 to reverce waveform.\nSelect 2 to amplify waveform.\nSelect 3 to produce a Tone Waveform.\nSelect 4 to process waveform through an IIR Filter.\nSelect 5 to PlayBack either the Input or the Output Sound File.\nSelect 6 to plot the waveform (You Must have installed GNUPLOT package for this opperation).\nSelect 7 to Record from Microphone to the Output Sound File.\nSelect anything else to quit.\n\n:>>");
		//scanf("%d",&opt1);
		
		
	
		switch (opt1){
		case 1:
		err=reverce_data ();
		break;
		case 2:
		//printf("\nEnter the amplification factor.\n:>>");
		//scanf("%lf",&k);
		err=amplify_data(k);
		break;
		case 3:
		fs=44100;
		//printf("\nThe value of sample rate is %lf Hz by default.\n",fs);
		//printf("Enter the frequency of the Tone in Hz.\n:>>");
		//scanf("%lf",&fc);
		//printf("\nEnter the Tones duration in seconds.\n:>>");
		//scanf("%lf",&Q);
		//printf("\nEnter the Tones Amplitude Vp. Maximum value is 2.\n:>>");
		//scanf("%lf",&k);
		chan=1;
		//printf("\nThe number of channels is %d by default.\n",chan);
		err=tone_create (fs, Q, k, fc, chan);
		break;
		case 4:
		//fs=44100;
		k=0; //Overall Gain in DB
		Q=0.707;
		//printf("\nSelect the filter type:\n1. For Low Pass Filter.\n2. For High Pass Filter.\n3. For Bandpass Filter.\n4. For Stopband Filter.\n5. For Notch Filter.\n6. For Low Shelf Filter.\n7. For High Shelf Filter\n:>>");
		//scanf("%d",&opt2);	
			
				switch(opt2){
					case 1:
					//printf("\nEnter the SAMPLERATE of the sound you want to Filter in KHz.\n:>>");
					//scanf("%lf",&fs); 
					//printf("\nThe value of samplerate is set to %lf KHz.\n",fs);
					//printf("Enter the cutoff frequency in Hz.\n:>>");
					//scanf("%lf",&fc);
					flt=BiQuad_new(LPF, k, fc,fs, Q);
					err=filter();
					stopmenu=1;
					break;
					case 2:
					//printf("\nEnter the SAMPLERATE of the sound you want to Filter in KHz.\n:>>");
					//scanf("%lf",&fs); 
					//printf("\nThe value of sample rate is set to %lf KHz.\n",fs);
					//printf("Enter the cutoff frequency in Hz.\n:>>");
					//scanf("%lf",&fc);
					flt=BiQuad_new(HPF, k, fc,fs, Q);
					err=filter();
					stopmenu=1;
					break;
					case 3:
					//printf("\nEnter the SAMPLERATE of the sound you want to Filter in KHz.\n:>>");
					//scanf("%lf",&fs); 
					//printf("\nThe value of sample rate is set to %lf KHz.\n",fs);
					//printf("Enter the Lower Cutoff frequency in Hz.\n:>>");
					//scanf("%lf",&fu);
					//printf("\nEnter the Higher Cutoff frequency in Hz.\n:>>");
					//scanf("%lf",&fl);
					err=bandpass_filter();
					stopmenu=1;
					break;
					case 4:
					//printf("\nEnter the SAMPLERATE of the sound you want to Filter in KHz.\n:>>");
					//scanf("%lf",&fs); 
					//printf("\nThe value of sample rate is set to %lf KHz.\n",fs);
					//printf("Enter the Lower Cutoff frequency in Hz.\n:>>");
					//scanf("%lf",&fl);
					//printf("\nEnter the Higher Cutoff frequency in Hz.\n:>>");
					//scanf("%lf",&fu);
					err=stopband_filter();
					stopmenu=1;
					break;
					case 5:
					//printf("\nEnter the SAMPLERATE of the sound you want to Filter in KHz.\n:>>");
					//scanf("%lf",&fs); 
					//printf("\nThe value of samplerate is set to %lf KHz.\n",fs);
					//printf("Enter the Central (Fc) frequency to cuttoff in Hz.\n:>>");
					//scanf("%lf",&fc);
					flt=BiQuad_new(NOTCH, k, fc,fs, Q);
					err=filter();
					stopmenu=1;
					break;
					case 6:
					//printf("\nEnter the SAMPLERATE of the sound you want to Filter in KHz.\n:>>");
					//scanf("%lf",&fs); 
					//printf("\nThe value of samplerate is set to %lf KHz.\n",fs);
					//printf("Enter the Fc frequency in Hz.\n:>>");
					//scanf("%lf",&fc);
					flt=BiQuad_new(LSH, k, fc,fs, Q);
					err=filter();
					stopmenu=1;
					break;
					case 7:
					//printf("\nEnter the SAMPLERATE of the sound you want to Filter in KHz.\n:>>");
					//scanf("%lf",&fs); 
					//printf("\nThe value of samplerate is set to %lf KHz.\n",fs);
					//printf("Enter the Fc frequency in Hz.\n:>>");
					//scanf("%lf",&fc);
					flt=BiQuad_new(HSH, k, fc,fs, Q);
					err=filter();
					stopmenu=1;
					break;
				}
		
		stopmenu=0;
		opt2=0;
		close_window3();
		break;
		case 5:
		//printf("\nSelect the sound file to play:\n1. For Input Sound File.\n2. For Output Sound File.\n:>>");
		//scanf("%d",&opt2);
		
if (stopplay==1){		
		#pragma omp parallel sections 
 		{ 
 		#pragma omp section 
 		{
		
		stopplay=0; //0 Do not stop play sound file. 1 Stop play sound file.
		err=playback(opt2);
		stopplay=1;
		sleep(1);	//for protection in case of very small file	
		close_window2();
		
		
		opt2=0;
		
		}
		#pragma omp section 
 		{
		
stop_window();

		}
		}
}
		break;
		case 6:
		//printf("\nSelect the sound file to plot waveform:\n1. For Input Sound File.\n2. For Output Sound File.\n:>>");
		//scanf("%d",&opt2);
		make_command_plot(opt2);
		system("sh $(pwd)/plot.sh");
		break;
		case 7:
		
		
		//printf("\nSelect the Duration in seconds of the recording.\n:>>");
		//scanf("%lf",&work);
		
if (stopplay==1){	
		#pragma omp parallel sections 
 		{ 
 		#pragma omp section 
 		{		
		stopplay=0; //0 Do not stop play sound file. 1 Stop play sound file.
		err=record_sound(work);
		stopplay=1;
		sleep(1);
		close_window2();
		work=0.0;
		opt2=0;
		}
		#pragma omp section 
 		{
		stop_window();
		}	
		}
				
}
		break;
		default :
		stopmenu=1;
	  gtk_text_buffer_get_iter_at_offset (buffer, &startiter, 0);
      gtk_text_buffer_get_iter_at_offset (buffer, &stopiter, -1);
      gtk_text_buffer_insert  (buffer, &startiter, "\nThis Program created by Panagiotis Georgiopoulos it07185", -1);
	  err=0;
	    }
	//}
print_message(message);
}

static int tone_create (double sr, double interval_sec, double amplitude, double frequency, int channels){
	//Produse a Tone
	
	sf_command (outfile, SFC_SET_CLIPPING, NULL, SF_TRUE);
    SNDFILE	*file;
	
	int	t;
	int	*buffer=NULL;
	
	if (amplitude>1.5){
		printf("Maximun allowed value for amplitude is 1.5 . Setting amplitude to 1.5\n");
		amplitude=1.5;
	}
	
	 
	amplitude *= 0x7F000000;
	amplitude /=2;
	frequency /= sr;
	
	t=sr*interval_sec*channels;
	if (! (buffer = new int[t])){
		printf ("allocation failed.\n") ;
		exit (0) ;
	}
	t=0;
	
	memset (&sfinfo, 0, sizeof (sfinfo));

	sfinfo.samplerate	= sr;
	sfinfo.frames		= sr*interval_sec;
	sfinfo.channels		= channels;
	sfinfo.format		= (SF_FORMAT_WAV | SF_FORMAT_PCM_24);


	if (sfinfo.channels == 1){
		
		for (t = 0 ; t < sfinfo.frames ; t++){
			buffer [t] = amplitude * sin (frequency * 2 * t * PI);
		}
		
					
	}else if (sfinfo.channels == 2){
		
		for (t = 0 ; t < sfinfo.frames ; t++){
			buffer [2 * t] = amplitude * sin (frequency * 2 * t * PI) ;
			buffer [2 * t + 1] = amplitude * sin (frequency * 2 * t * PI) ;
		}
		
	}else{
		
		printf ("makesine can only generate mono or stereo files.\n");
		exit (1);
		
	}

	if (! (file = sf_open (outfilename , SFM_WRITE, &sfinfo))){
		printf ("Error : Not able to open output file.\n");
		return 1;
	}
	
	
	if (sf_write_int (file, buffer, channels*sr*interval_sec) == channels*sr*interval_sec){
		//puts (sf_strerror (file));
	}else{
		printf("File was not writen\n");
	}
	
	delete [] buffer;
		buffer = NULL;	
	sf_close (file);
	
	//END Produce Tone
	return 0 ;
}

static int amplify_data (double gain){
	//Amplify waveform without clipping
	
	
	double channel_gain [MAX_CHANNELS] = { gain, gain, gain, gain, gain, gain } ;
    int k, chan ;
	
	if (! (infile = sf_open (infilename, SFM_READ, &sfinfo))) {  
        printf ("Not able to open input file %s.\n", infilename) ;
        puts (sf_strerror (NULL)) ;
        return 1 ;
        }

    if (sfinfo.channels > MAX_CHANNELS){
		printf ("Not able to process more than %d channels\n", MAX_CHANNELS) ;
        return 1 ;
        }	
		
	if (! (outfile = sf_open (outfilename, SFM_WRITE, &sfinfo))){
		printf ("Not able to open output file %s.\n", outfilename) ;
        puts (sf_strerror (NULL)) ;
        return 1 ;
        }
    sf_command (outfile, SFC_SET_CLIPPING, NULL, SF_TRUE);
    while ((readcount = sf_read_double (infile, data, BUFFER_LEN))){
    for (chan = 0 ; chan < sfinfo.channels ; chan ++){
        for (k = chan ; k < readcount ; k+= sfinfo.channels){
            data [k] *= channel_gain [chan] ;
		}
	}
	sf_write_double (outfile, data, readcount);
	}
	
	sf_close (infile);
	sf_close (outfile);
	
	//END Amplify
	
    return 0;
} 

static int reverce_data (){
	
	//Reverce waveform
	sf_command (outfile, SFC_SET_CLIPPING, NULL, SF_FALSE);		
	long all_count_data,i,k;
	
	
	
	if (! (infile = sf_open (infilename, SFM_READ, &sfinfo))) {  
        printf ("Not able to open input file %s.\n", infilename) ;
        puts (sf_strerror (NULL)) ;
        return 1 ;
        }

    if (sfinfo.channels > MAX_CHANNELS){
		printf ("Not able to process more than %d channels\n", MAX_CHANNELS) ;
        return 1 ;
        }
		
		     	
		all_count_data=sfinfo.frames*sfinfo.channels;
			
		double *tempdata = NULL;
		tempdata = new double[all_count_data];
					
		i=0;
		while ((readcount = sf_read_double (infile, data, BUFFER_LEN))){
						
				for (k = 0 ; k <readcount ; k++){
            	tempdata[all_count_data-i-1]=data[k];
				i++;
					
				}
		}
						
		if (! (outfile = sf_open (outfilename, SFM_WRITE, &sfinfo))){
		printf ("Not able to open output file %s.\n", outfilename) ;
        puts (sf_strerror (NULL)) ;
        return 1 ;
        }
		sf_write_double (outfile, tempdata, all_count_data) ;
			
		
		delete [] tempdata;
		tempdata = NULL;
		//end reverce waveform
    
	sf_close (infile);
	sf_close (outfile);
	
    return 0;
} 


int bandpass_filter(){
	sf_command (outfile, SFC_SET_CLIPPING, NULL, SF_TRUE);
    
	
	int	i,channels,j;
	float sr;
	float *bufferin=NULL;
	float *bufferout=NULL;
	long frames,all_count_data,chunk;
		
	if (! (infile = sf_open (infilename, SFM_READ, &sfinfo))) {  
        printf ("Not able to open input file %s.\n", infilename) ;
        puts (sf_strerror (NULL)) ;
        return 1 ;
        }
	channels=sfinfo.channels;
	sr=sfinfo.samplerate;	
	all_count_data=sfinfo.frames*sfinfo.channels;	
	frames=sfinfo.frames;
		
	if (! (outfile = sf_open (outfilename, SFM_WRITE, &sfinfo))){
		printf ("Not able to open output file %s.\n", outfilename) ;
        puts (sf_strerror (NULL)) ;
        return 1 ;
        }
		
	if (! (bufferin = new float[all_count_data])){
		printf ("allocation failed.\n") ;
		exit (0) ;
	}
	
	if (! (bufferout = new float[all_count_data])){
		printf ("allocation failed.\n") ;
		exit (0) ;
	}
	
	flt=BiQuad_new(HPF, k, fl,fs, Q);	
	secondary_flt=BiQuad_new(LPF, k, fu,fs, Q);
	
	readcount = sf_readf_float (infile, bufferin, frames);
	chunk=ceil(frames/nthreads);
		
omp_set_num_threads(nthreads);
omp_set_nested(0);
omp_init_lock(&lock);
	//while ((readcount = sf_readf_float (infile, bufferin, BUFFER_LEN)) > 0){
#pragma omp parallel private(tid,i,j) shared(chunk,readcount,channels,flt,secondary_flt,bufferout,bufferin) 		
{
tid=omp_get_thread_num();
	biquad tempflt;
	tempflt.a0=flt->a0;
	tempflt.a1=flt->a1;
	tempflt.a2=flt->a2;
	tempflt.a3=flt->a3;
	tempflt.a4=flt->a4;
	tempflt.y1=0;
	if(tid>0){
	tempflt.x1=bufferin[tid*chunk-3];
	tempflt.x2=bufferin[tid*chunk-4];
	tempflt.y1=BiQuad(bufferin[tid*chunk-2], &tempflt);
	tempflt.x1=bufferin[tid*chunk-2];
	tempflt.x2=bufferin[tid*chunk-3];
	tempflt.y2=BiQuad(bufferin[tid*chunk-1], &tempflt);
	}
	biquad tempSecondary_flt;
	tempSecondary_flt.a0=secondary_flt->a0;
	tempSecondary_flt.a1=secondary_flt->a1;
	tempSecondary_flt.a2=secondary_flt->a2;
	tempSecondary_flt.a3=secondary_flt->a3;
	tempSecondary_flt.a4=secondary_flt->a4;
	tempSecondary_flt.y1=0;
	if(tid>0){
	tempSecondary_flt.x1=0;
	tempSecondary_flt.x2=0;
	tempSecondary_flt.y1=BiQuad(bufferin[tid*chunk-2], &tempSecondary_flt);
	tempSecondary_flt.x1=tempflt.y1;
	tempSecondary_flt.x2=tempflt.y2;
	tempSecondary_flt.y2=BiQuad(bufferin[tid*chunk-1], &tempSecondary_flt);
	}
#pragma omp barrier

#pragma omp for schedule(static,chunk) nowait			
		for (i = 0 ; i < readcount ; i++){
			for (j = 0 ; j < channels ; j++){
				bufferout[i * channels + j]=BiQuad(bufferin[i * channels + j], &tempflt);
				bufferout[i * channels + j]=BiQuad(bufferout[i * channels + j], &tempSecondary_flt);				
			}
		}
}
		sf_writef_float(outfile,bufferout,frames);
	//}
omp_destroy_lock(&lock);
omp_set_num_threads(nthreads);
	
	delete [] bufferin;
	delete [] bufferout;
	bufferin=NULL;
	bufferout=NULL;
	
	sf_close (infile);
	sf_close (outfile);
		
	return 0;
}

int stopband_filter(){
	sf_command (outfile, SFC_SET_CLIPPING, NULL, SF_TRUE);
    
	
	int	i,channels,j;
	float sr;
	float *bufferin=NULL;
	float *bufferout=NULL;
	long frames,all_count_data,chunk;
		
	if (! (infile = sf_open (infilename, SFM_READ, &sfinfo))) {  
        printf ("Not able to open input file %s.\n", infilename) ;
        puts (sf_strerror (NULL)) ;
        return 1 ;
        }
	channels=sfinfo.channels;
	sr=sfinfo.samplerate;
	all_count_data=sfinfo.frames*sfinfo.channels;	
	frames=sfinfo.frames;
		
	if (! (outfile = sf_open (outfilename, SFM_WRITE, &sfinfo))){
		printf ("Not able to open output file %s.\n", outfilename) ;
        puts (sf_strerror (NULL)) ;
        return 1 ;
        }
		
	if (! (bufferin = new float[all_count_data])){
		printf ("allocation failed.\n") ;
		exit (0) ;
	}
	
	if (! (bufferout = new float[all_count_data])){
		printf ("allocation failed.\n") ;
		exit (0) ;
	}
	
	flt=BiQuad_new(LPF, k, fl,fs, Q);	
	secondary_flt=BiQuad_new(HPF, k, fu,fs, Q);
	//while ((readcount = sf_readf_float (infile, bufferin, BUFFER_LEN)) > 0){
	readcount = sf_readf_float (infile, bufferin, frames);
	chunk=ceil(frames/nthreads);
		
omp_set_num_threads(nthreads);
omp_set_nested(0);
omp_init_lock(&lock);
#pragma omp parallel private(tid,i,j) shared(chunk,readcount,channels,flt,secondary_flt,bufferout,bufferin) 		
{
tid=omp_get_thread_num();
	biquad tempflt;
	tempflt.a0=flt->a0;
	tempflt.a1=flt->a1;
	tempflt.a2=flt->a2;
	tempflt.a3=flt->a3;
	tempflt.a4=flt->a4;
	tempflt.y1=0;
	if(tid>0){
	tempflt.x1=bufferin[tid*chunk-3];
	tempflt.x2=bufferin[tid*chunk-4];
	tempflt.y1=BiQuad(bufferin[tid*chunk-2], &tempflt);
	tempflt.x1=bufferin[tid*chunk-2];
	tempflt.x2=bufferin[tid*chunk-3];
	tempflt.y2=BiQuad(bufferin[tid*chunk-1], &tempflt);
	}
	biquad tempSecondary_flt;
	tempSecondary_flt.a0=secondary_flt->a0;
	tempSecondary_flt.a1=secondary_flt->a1;
	tempSecondary_flt.a2=secondary_flt->a2;
	tempSecondary_flt.a3=secondary_flt->a3;
	tempSecondary_flt.a4=secondary_flt->a4;
	tempSecondary_flt.y1=0;
	if(tid>0){
	tempSecondary_flt.x1=bufferin[tid*chunk-3];
	tempSecondary_flt.x2=bufferin[tid*chunk-4];
	tempSecondary_flt.y1=BiQuad(bufferin[tid*chunk-2], &tempSecondary_flt);
	tempSecondary_flt.x1=bufferin[tid*chunk-2];
	tempSecondary_flt.x2=bufferin[tid*chunk-3];
	tempSecondary_flt.y2=BiQuad(bufferin[tid*chunk-1], &tempSecondary_flt);
	}
#pragma omp barrier

#pragma omp for schedule(static,chunk) nowait	
		for (i = 0 ; i < readcount ; i++){
			for (j = 0 ; j < channels ; j++){
				bufferout[i * channels + j]=BiQuad(bufferin[i * channels + j], &tempflt);
				bufferout[i * channels + j]-=BiQuad(bufferin[i * channels + j], &tempSecondary_flt);				
			}
		}
}
		sf_writef_float(outfile,bufferout,frames);
	//}
omp_destroy_lock(&lock);
omp_set_num_threads(nthreads);
	
	delete [] bufferin;
	delete [] bufferout;
	bufferin=NULL;
	bufferout=NULL;
	
	sf_close (infile);
	sf_close (outfile);
		
	return 0;
}

static int filter(){
	sf_command (outfile, SFC_SET_CLIPPING, NULL, SF_TRUE);
    
	
	int	i,channels,j;
	float sr;
	float *bufferin=NULL;
	float *bufferout=NULL;
	long all_count_data,chunk,frames;
		
	if (! (infile = sf_open (infilename, SFM_READ, &sfinfo))) {  
        printf ("Not able to open input file %s.\n", infilename) ;
        puts (sf_strerror (NULL)) ;
        return 1 ;
        }
	channels=sfinfo.channels;
	sr=sfinfo.samplerate;	
	all_count_data=sfinfo.frames*sfinfo.channels;	
	frames=sfinfo.frames;	
	if (! (outfile = sf_open (outfilename, SFM_WRITE, &sfinfo))){
		printf ("Not able to open output file %s.\n", outfilename) ;
        puts (sf_strerror (NULL)) ;
        return 1 ;
        }
		
	if (! (bufferin = new float[all_count_data])){
		printf ("allocation failed.\n") ;
		exit (0) ;
	}
	
	if (! (bufferout = new float[all_count_data])){
		printf ("allocation failed.\n") ;
		exit (0) ;
	}
	
	
chunk=ceil(frames/nthreads);	
omp_set_num_threads(nthreads);
omp_set_nested(0);
omp_init_lock(&lock);
//while ((readcount = sf_readf_float (infile, bufferin, sfinfo.frames)) > 0){	
readcount = sf_readf_float (infile, bufferin, frames);

#pragma omp parallel private(i,j,tid) shared(chunk,channels,bufferout,bufferin,readcount,flt) num_threads(nthreads)
{	
tid=omp_get_thread_num();	
	
biquad tempflt;
tempflt.a0=flt->a0;
tempflt.a1=flt->a1;
tempflt.a2=flt->a2;
tempflt.a3=flt->a3;
tempflt.a4=flt->a4;
tempflt.y1=0;
if(tid>0){
tempflt.x1=bufferin[tid*chunk-3];
tempflt.x2=bufferin[tid*chunk-4];
tempflt.y1=BiQuad(bufferin[tid*chunk-2], &tempflt);
tempflt.x1=bufferin[tid*chunk-2];
tempflt.x2=bufferin[tid*chunk-3];
tempflt.y2=BiQuad(bufferin[tid*chunk-1], &tempflt);
}

#pragma omp barrier

#pragma omp for schedule(static,chunk) nowait
		for (i = 0 ; i <readcount ;i++){
			
			for (j = 0 ; j < channels ; j++){
				bufferout[i * channels + j]=BiQuad(bufferin[i * channels + j], &tempflt);	
						
			}
									
		}

}	
		
		sf_writef_float(outfile,bufferout,frames);
//	}
omp_destroy_lock(&lock);
omp_set_num_threads(nthreads);
	
	delete [] bufferin;
	delete [] bufferout;
	
	bufferin=NULL;
	bufferout=NULL;
	
	sf_close (infile);
	sf_close (outfile);
	return 0;
}

/* Below this would be biquad.c */
/* Computes a BiQuad filter on a sample */
smp_type BiQuad(smp_type sample, biquad * b)
{
smp_type result;
/* compute result */
result = b->a0 * sample + b->a1 * b->x1 + b->a2 * b->x2 -
b->a3 * b->y1 - b->a4 * b->y2;
/* shift x1 to x2, sample to x1 */
b->x2 = b->x1;
b->x1 = sample;
/* shift y1 to y2, result to y1 */
b->y2 = b->y1;
b->y1 = result;
return result;
}


/* sets up a BiQuad Filter */
biquad *BiQuad_new(int type, smp_type dbGain, smp_type freq,smp_type srate, smp_type q)
{
biquad *b= NULL;
b= new biquad[sizeof(biquad)];
smp_type A, omega, sn, cs, alpha, beta;
smp_type a0, a1, a2, b0, b1, b2;
//b = malloc(sizeof(biquad));
if (b == NULL){
return NULL;
printf("\n b was null");
}
/* setup variables */
A = pow(10, dbGain /40);
omega = 2 * M_PI * freq /srate;
sn = sin(omega);
cs = cos(omega);
alpha = sn/(2*Q);
beta = sqrt(A + A);
switch (type) {
case LPF:
b0 = (1 - cs) /2;
b1 = 1 - cs;
b2 = (1 - cs) /2;
a0 = 1 + alpha;
a1 = -2 * cs;
a2 = 1 - alpha;
break;
case HPF:
b0 = (1 + cs) /2;
b1 = -(1 + cs);
b2 = (1 + cs) /2;
a0 = 1 + alpha;
a1 = -2 * cs;
a2 = 1 - alpha;
break;
case BPF:
b0 = alpha;
b1 = 0;
b2 = -alpha;
a0 = 1 + alpha;
a1 = -2 * cs;
a2 = 1 - alpha;
break;
case NOTCH:
b0 = 1;
b1 = -2 * cs;
b2 = 1;
a0 = 1 + alpha;
a1 = -2 * cs;
a2 = 1 - alpha;
break;
case PEQ:
b0 = 1 + (alpha * A);
b1 = -2 * cs;
b2 = 1 - (alpha * A);
a0 = 1 + (alpha /A);
a1 = -2 * cs;
a2 = 1 - (alpha /A);
break;
case LSH:
b0 = A * ((A + 1) - (A - 1) * cs + beta * sn);
b1 = 2 * A * ((A - 1) - (A + 1) * cs);
b2 = A * ((A + 1) - (A - 1) * cs - beta * sn);
a0 = (A + 1) + (A - 1) * cs + beta * sn;
a1 = -2 * ((A - 1) + (A + 1) * cs);
a2 = (A + 1) + (A - 1) * cs - beta * sn;
break;
case HSH:
b0 = A * ((A + 1) + (A - 1) * cs + beta * sn);
b1 = -2 * A * ((A - 1) + (A + 1) * cs);
b2 = A * ((A + 1) + (A - 1) * cs - beta * sn);
a0 = (A + 1) - (A - 1) * cs + beta * sn;
a1 = 2 * ((A - 1) - (A + 1) * cs);
a2 = (A + 1) - (A - 1) * cs - beta * sn;
break;
default:
delete [] b;
b=NULL;
return NULL;
}
/* precompute the coefficients */
printf("\n\nThe filter coefficients are:\n");
b->a0 = b0 /a0;
printf("%lf\n",b->a0);
b->a1 = b1 /a0;
printf("%lf\n",b->a1);
b->a2 = b2 /a0;
printf("%lf\n",b->a2);
b->a3 = a1 /a0;
printf("%lf\n",b->a3);
b->a4 = a2 /a0;
printf("%lf\n",b->a4);
/* zero initial samples */
b->x1 = b->x2 = 0;
b->y1 = b->y2 = 0;
return b;
}


int playback(int opt){
	 
	 int fd,status,k,arg;	/* sound device file descriptor */
	 short *buffer=NULL;
	 	 
	fd = open("/dev/dsp", O_RDWR);
 	if (fd < 0) {
		strcpy(command,"");
		strcpy(command,"play ");
			if (opt==1 && infilename){
				strcpy(&command[strlen(command)],infilename);
			}
			if (opt==2 && outfilename){
				strcpy(&command[strlen(command)],outfilename);
			}
		system(command); 
		
		return 0;
  	}
	
	if (opt==1){
		if (! (infile = sf_open (infilename, SFM_READ, &sfinfo))) {  
        	//printf ("Not able to open input file %s.\n", infilename) ;
        	//puts (sf_strerror (NULL)) ;
        	//return 1;
        }
	}else if (opt==2){
		if (! (infile = sf_open (outfilename, SFM_READ, &sfinfo))) {  
        	//printf ("Not able to open input file %s.\n", infilename) ;
        	//puts (sf_strerror (NULL)) ;
        	//return 1;
        }	
	}else{
		//printf("\nInvalid option for PlayBack Function. Must be 1 OR 2.\n");
		//return 1;
	}

    if (sfinfo.channels > MAX_CHANNELS){
		//printf ("Not able to process more than %d channels\n", MAX_CHANNELS) ;
        //return 1 ;
        }
	
	 /* set sampling parameters */
  arg = 16;	   /* sample size */
  status = ioctl(fd, SOUND_PCM_WRITE_BITS, &arg);
  if (status == -1){
   // perror("SOUND_PCM_WRITE_BITS ioctl failed");
	}
 

  
   arg = sfinfo.channels;  /* mono or stereo */
  status = ioctl(fd, SOUND_PCM_WRITE_CHANNELS, &arg);
  if (status == -1){
   // perror("SOUND_PCM_WRITE_CHANNELS ioctl failed");
  }
  
  arg = sfinfo.samplerate;	   /* sampling rate */
  status = ioctl(fd, SOUND_PCM_WRITE_RATE, &arg);
  if (status == -1){
    //perror("SOUND_PCM_WRITE_WRITE ioctl failed");
  }
			
	if (! (buffer = new short[sfinfo.samplerate])){
		//printf ("allocation failed.\n") ;
		//exit (0) ;
	}	
		
	status = ioctl(fd, SOUND_PCM_SYNC, 0);
    sf_command (outfile, SFC_SET_CLIPPING, NULL, SF_TRUE);
    while(readcount = sf_read_short (infile, buffer, sfinfo.samplerate)){
    	if (stopplay==0){
			status = write(fd, buffer, sfinfo.samplerate*sizeof(short)); /* play it back */
		}else if (stopplay==1){
			break;
		}else{
			//printf("\nInvaliv value of variable stopplay. Must be 0 OR 1. Stopping\n");
			break;
		}
		for (k=0;k<sfinfo.samplerate;k++){
			buffer[k]=0;
		}		
		
	}
	status = ioctl(fd, SOUND_PCM_SYNC, 0);
	
	sf_close (infile);
	close(fd);
	delete [] buffer;
	buffer = NULL;
	stopplay=1;
	
	
    return 0;		
}



int record_sound(float length){
	
	int fd,status,arg;	/* sound device file descriptor */
	long k;
	short *buffer=NULL;
	//fd = open("/dev/dsp", O_RDWR);
	fd=-1;
	if (fd < 0) {
	 // print_message("\nopen of /dev/dsp failed. \nStarting alternative recorting method using rec command.\nUse stop button to stop recording");
   		
   		strcpy(command,"");
		strcpy(command,"rec -b 16 -r 44100 ");
		
		strcpy(&command[strlen(command)],outfilename);
		system(command); 
		
   		//perror("open of /dev/dsp failed");
    	return 0;
  	}
		 /* set sampling parameters */
  arg = 16;	   /* sample size */
  status = ioctl(fd, SOUND_PCM_WRITE_BITS, &arg);
  if (status == -1){
	 // print_message("SOUND_PCM_WRITE_BITS ioctl failed");
    //perror("SOUND_PCM_WRITE_BITS ioctl failed");
	}

   arg = 1;  /* mono or stereo */
  status = ioctl(fd, SOUND_PCM_WRITE_CHANNELS, &arg);
  if (status == -1){
	 // print_message("SOUND_PCM_WRITE_CHANNELS ioctl failed");
    //perror("SOUND_PCM_WRITE_CHANNELS ioctl failed");
  }
  
  arg = 44100;	   /* sampling rate */
  status = ioctl(fd, SOUND_PCM_WRITE_RATE, &arg);
  if (status == -1){
	 // print_message("SOUND_PCM_WRITE_WRITE ioctl failed");
   // perror("SOUND_PCM_WRITE_WRITE ioctl failed");
  }
	
	k=length*44100*16*1/8;		
	if (! (buffer = new short[k])){
		
		//print_message ("allocation failed.\n") ;
		//exit (0) ;
	}	
		
	status = ioctl(fd, SOUND_PCM_SYNC, 0);
    sf_command (outfile, SFC_SET_CLIPPING, NULL, SF_TRUE);
	
	//print_message("\nRecording........\n");
	//printf("\nRecording........\n");
	status = read(fd, buffer, k); /* record some sound */
    if (status != k){
    	//perror("read wrong number of bytes");
    }
	//print_message("\nSaving to output file........\n");
	//printf("\nSaving to output file........\n");
	
	memset (&sfinfo, 0, sizeof (sfinfo));

	sfinfo.samplerate	= 44100;
	sfinfo.frames		= k;
	sfinfo.channels		= 1;
	sfinfo.format		= (SF_FORMAT_WAV | SF_FORMAT_PCM_24);

	
	if (! (outfile = sf_open (outfilename, SFM_WRITE, &sfinfo))){
		//print_message ("Not able to open the output file \n") ;
        //puts (sf_strerror (NULL)) ;
        return 1 ;
    }

	sf_writef_short(outfile,buffer,k/2);
	status = ioctl(fd, SOUND_PCM_SYNC, 0);
	
	status = ioctl(fd, SOUND_PCM_SYNC, 0);
	delete [] buffer;
	buffer = NULL;
	close(fd);
	sf_close (outfile);
	
	return 0;
}


int make_command_plot(int opt){
	
	FILE *ofile=NULL;
			
		if ((ofile = fopen ("plot.sh", "w")) == NULL){
			//printf ("Not able to open output file %s : %s\n", "gp_datain.gp", sf_strerror (NULL)) ;
			//return 1 ;
		} 
	
	    if (opt==1){
			fprintf (ofile, "#!/bin/sh\n\n$(pwd)/plot_data 1 %s",infilename);
		}else if (opt==2){
			fprintf (ofile, "#!/bin/sh\n\n$(pwd)/plot_data 2 %s",outfilename);
		}
				
	fclose (ofile) ;
		
	return 0;
}
