#include <iostream>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <sndfile.h>
using namespace std;

#define	BUFFER_LEN	1024
#define MAX_CHANNELS	6
#define	PI	3.14159265358979323846264338
#define FILTER_SECTIONS   2  /* 2 filter sections for 24 db/oct filter */

typedef struct {
    unsigned int length;       /* size of filter */
    float *history;            /* pointer to history in filter */
    float *coef;               /* pointer to coefficients of filter */
} FILTER;


void mainmenu();
static int tone_create (double sr, double interval_sec, double amplitude, double frequency, int channels);
static int amplify_data (double gain);
static int reverce_data ();
void prewarp(double *a0, double *a1, double *a2, double fc, double fs);
void bilinear(
    double a0, double a1, double a2,    /* numerator coefficients */
    double b0, double b1, double b2,    /* denominator coefficients */
    double *k,                                   /* overall gain factor */
    double fs,                                   /* sampling rate */
    float *coef);                         /* pointer to 4 iir coefficients */
void szxform(
    double *a0, double *a1, double *a2, /* numerator coefficients */
    double *b0, double *b1, double *b2, /* denominator coefficients */
    double fc,         /* Filter cutoff frequency */
    double fs,         /* sampling rate */
    double *k,         /* overall gain factor */
    float *coef);         /* pointer to 4 iir coefficients */
	
int create_lowpass_iir_filter();
float iir_filter(float input,FILTER *iir);
static int filter();



/* FILTER INFORMATION STRUCTURE FOR FILTER ROUTINES */


typedef struct {
        double a0, a1, a2;       /* numerator coefficients */
        double b0, b1, b2;       /* denominator coefficients */
} BIQUAD;

BIQUAD ProtoCoef[FILTER_SECTIONS];      /* Filter prototype coefficients,
                                                     1 for each filter section */

void szxform(
    double *a0, double *a1, double *a2,     /* numerator coefficients */
    double *b0, double *b1, double *b2,   /* denominator coefficients */
    double fc,           /* Filter cutoff frequency */
    double fs,           /* sampling rate */
    double *k,           /* overall gain factor */
    float *coef);         /* pointer to 4 iir coefficients */

 FILTER   iir;
 float    *coef;
 double   fs, fc;     /* Sampling frequency, cutoff frequency OR Sampling frequency, frequency of Tone*/
 double   k;           /* overall gain factor OR amplitude of Tone OR The amplification factor*/
 double   Q;     /* Resonance > 1.0 < 1000 OR interval of Tone*/



static double data [BUFFER_LEN];
SNDFILE *infile, *outfile;
SF_INFO sfinfo ;
SF_FORMAT_INFO info;

int	readcount, err,opt,chan,stopmenu;

char infilename[300];
char outfilename[300];

int main (){
	
err=1;

mainmenu();
	
return err;
}

void mainmenu(){
	
	stopmenu=0;
   
		printf("Enter the name and the path of the sound file to be processed, input file *.wav .\n:>>");
		scanf("%s",infilename);
		printf("Enter the name and the path of the output file *.wav .\n:>>");
		scanf("%s",outfilename);
	while(!stopmenu){
		printf("\n");
		printf("Select 1 to reverce waveform.\nSelect 2 to amplify waveform.\nSelect 3 to produce a Tone Waveform.\nSelect 4 to process waveform through an IIR Low Pass Filter.\nSelect anything else to quit.\n\n:>>");
		scanf("%d",&opt);
		
		
	
		switch (opt){
		case 1:
		err=reverce_data ();
		break;
		case 2:
		printf("\nEnter the amplification factor.\n:>>");
		scanf("%lf",&k);
		err=amplify_data(k);
		break;
		case 3:
		fs=44100;
		printf("\nThe value of sample rate is %lf Hz by default.\n",fs);
		printf("Enter the frequency of the Tone in Hz.\n:>>");
		scanf("%lf",&fc);
		printf("\nEnter the Tones duration in seconds.\n:>>");
		scanf("%lf",&Q);
		printf("\nEnter the Tones Amplitude Vp. Maximum value is 2.\n:>>");
		scanf("%lf",&k);
		chan=1;
		printf("\nThe number of channels is %d by default.\n",chan);
		err=tone_create (fs, Q, k, fc, chan);
		break;
		case 4:
		fs=44100;
		printf("\nThe value of sample rate is %lf Hz by default.\n",fs);
		printf("Enter the cutoff frequency in Hz.\n:>>");
		scanf("%lf",&fc);
		k=1.0;
		Q=1;
		err=create_lowpass_iir_filter();
		err=filter();
		break;
		default :
		stopmenu=1;
		printf("\n.........\nThis Program created by Panagiotis Georgiopoulos it07185\n\n");
		err=0;
	    }
	}
	
}

static int tone_create (double sr, double interval_sec, double amplitude, double frequency, int channels){
	//Produse a Tone
	
	sf_command (outfile, SFC_SET_CLIPPING, NULL, SF_TRUE);
    SNDFILE	*file;
	
	int	k;
	int	*buffer=NULL;
	
	if (amplitude>2){
		printf("Maximun allowed value for amplitude is 2. Setting amplitude to 2\n");
		amplitude=2;
	}
	
	amplitude *= 0x7F000000;
	amplitude /=2;
	frequency /= sr;
	
	k=sr*interval_sec*channels;
	if (! (buffer = new int[k])){
		printf ("allocation failed.\n") ;
		exit (0) ;
	}
	k=0;
	
	memset (&sfinfo, 0, sizeof (sfinfo));

	sfinfo.samplerate	= sr;
	sfinfo.frames		= sr*interval_sec;
	sfinfo.channels		= channels;
	sfinfo.format		= (SF_FORMAT_WAV | SF_FORMAT_PCM_24);


	if (sfinfo.channels == 1){
		
		for (k = 0 ; k < sfinfo.frames ; k++){
			buffer [k] = amplitude * sin (frequency * 2 * k * PI);
		}
		
					
	}else if (sfinfo.channels == 2){
		
		for (k = 0 ; k < sfinfo.frames ; k++){
			buffer [2 * k] = amplitude * sin (frequency * 2 * k * PI) ;
			buffer [2 * k + 1] = amplitude * sin (frequency * 2 * k * PI) ;
		}
		
	}else{
		
		printf ("makesine can only generate mono or stereo files.\n");
		exit (1);
		
	}

	if (! (file = sf_open (outfilename , SFM_WRITE, &sfinfo))){
		printf ("Error : Not able to open output file.\n");
		return 1;
	}
	
	
	if (sf_write_int (file, buffer, channels*sr*interval_sec) == channels*sr*interval_sec){
		//puts (sf_strerror (file));
	}else{
		printf("File was not writen\n");
	}
	
	delete [] buffer;
		buffer = NULL;	
	sf_close (file);
	
	//END Produce Tone
	return 0 ;
}

static int amplify_data (double gain){
	//Amplify waveform without clipping
	
	
	double channel_gain [MAX_CHANNELS] = { gain, gain, gain, gain, gain, gain } ;
    int k, chan ;
	
	if (! (infile = sf_open (infilename, SFM_READ, &sfinfo))) {  
        printf ("Not able to open input file %s.\n", infilename) ;
        puts (sf_strerror (NULL)) ;
        return 1 ;
        }

    if (sfinfo.channels > MAX_CHANNELS){
		printf ("Not able to process more than %d channels\n", MAX_CHANNELS) ;
        return 1 ;
        }	
		
	if (! (outfile = sf_open (outfilename, SFM_WRITE, &sfinfo))){
		printf ("Not able to open output file %s.\n", outfilename) ;
        puts (sf_strerror (NULL)) ;
        return 1 ;
        }
    sf_command (outfile, SFC_SET_CLIPPING, NULL, SF_TRUE);
    while ((readcount = sf_read_double (infile, data, BUFFER_LEN))){
    for (chan = 0 ; chan < sfinfo.channels ; chan ++){
        for (k = chan ; k < readcount ; k+= sfinfo.channels){
            data [k] *= channel_gain [chan] ;
		}
	}
	sf_write_double (outfile, data, readcount);
	}
	
	sf_close (infile);
	sf_close (outfile);
	
	//END Amplify
	
    return 0;
} 

static int reverce_data (){
	
	//Reverce waveform
	sf_command (outfile, SFC_SET_CLIPPING, NULL, SF_FALSE);		
	long all_count_data,offset,i,k,m;
	
	
	
	if (! (infile = sf_open (infilename, SFM_READ, &sfinfo))) {  
        printf ("Not able to open input file %s.\n", infilename) ;
        puts (sf_strerror (NULL)) ;
        return 1 ;
        }

    if (sfinfo.channels > MAX_CHANNELS){
		printf ("Not able to process more than %d channels\n", MAX_CHANNELS) ;
        return 1 ;
        }
		
		     	
		all_count_data=sfinfo.frames*sfinfo.channels;
			
		double *tempdata = NULL;
		tempdata = new double[all_count_data];
					
		i=0;
		offset=0;
		
		while ((readcount = sf_read_double (infile, data, BUFFER_LEN))){
			offset+=readcount;
			
				for (k = 0 ; k <readcount ; k++){
            	tempdata[all_count_data-i-1]=data[k];
					if(i!=all_count_data){
					i++;
					} 
				}
			
				
			i=offset;
			     
        }
						
		if (! (outfile = sf_open (outfilename, SFM_WRITE, &sfinfo))){
		printf ("Not able to open output file %s.\n", outfilename) ;
        puts (sf_strerror (NULL)) ;
        return 1 ;
        }
		sf_write_double (outfile, tempdata, all_count_data) ;
			
		
		delete [] tempdata;
		tempdata = NULL;
		//end reverce waveform
    
	sf_close (infile);
	sf_close (outfile);
	
    return 0;
} 

static int filter(){
	sf_command (outfile, SFC_SET_CLIPPING, NULL, SF_FALSE);
    
	
	int	k,channels,m;
	float sr;
	float *bufferin=NULL;
	float *bufferout=NULL;
	
	if (! (infile = sf_open (infilename, SFM_READ, &sfinfo))) {  
        printf ("Not able to open input file %s.\n", infilename) ;
        puts (sf_strerror (NULL)) ;
        return 1 ;
        }
	channels=sfinfo.channels;
	sr=sfinfo.samplerate;	
		
	if (! (outfile = sf_open (outfilename, SFM_WRITE, &sfinfo))){
		printf ("Not able to open output file %s.\n", outfilename) ;
        puts (sf_strerror (NULL)) ;
        return 1 ;
        }
		
	if (! (bufferin = new float[BUFFER_LEN*channels])){
		printf ("allocation failed.\n") ;
		exit (0) ;
	}
	
	if (! (bufferout = new float[BUFFER_LEN*channels])){
		printf ("allocation failed.\n") ;
		exit (0) ;
	}
	
		
	while ((readcount = sf_readf_float (infile, bufferin, BUFFER_LEN)) > 0){
		for (k = 0 ; k < readcount ; k++){
			for (m = 0 ; m < channels ; m++){
				bufferout[k * channels + m]=iir_filter(bufferin[k * channels + m], &iir);
			}
		}
		sf_writef_float(outfile,bufferout,BUFFER_LEN);
	}
	
	delete [] bufferin;
	delete [] bufferout;
	bufferin=NULL;
	bufferout=NULL;
	
	sf_close (infile);
	sf_close (outfile);
	return 0;
}


int create_lowpass_iir_filter(){
		
 	unsigned nInd;
 	double   a0, a1, a2, b0, b1, b2;
	double alpha,cs,sn,omega;
	
	 iir.length = FILTER_SECTIONS;         /* Number of filter sections */

/*
 * Allocate array of z-domain coefficients for each filter section
 * plus filter gain variable
 */
        iir.coef = (float *) calloc(4 * iir.length + 1, sizeof(float));
		if (!iir.coef)
        {
                 printf("Unable to allocate coef array, exiting\n");
                 exit(1);
        }
			
	
		if(!k){ 
        k = 1.0;          /* Set overall filter gain */
		}
		
        coef = iir.coef+1;     /* Skip k, or gain */

		if(!Q){ 
        Q = 1;
		}
		if(!fc){                         /* Resonance */
        fc = 3000; /* Filter cutoff (Hz) */
		}  
		if(!fs){                
        fs = 44100;                      /* Sampling frequency (Hz) */
		}
		
	
	
		/*
 * Setup filter s-domain coefficients for Low Pass Filter
 */
                 /* Section 1 */       
        ProtoCoef[0].a0 = 1.0;
        ProtoCoef[0].a1 = 0;
        ProtoCoef[0].a2 = 0;
        ProtoCoef[0].b0 = 1.0;
        ProtoCoef[0].b1 = 0.765367;
        ProtoCoef[0].b2 = 1.0;

                 /* Section 2 */ 
				    
        ProtoCoef[1].a0 = 1.0;
        ProtoCoef[1].a1 = 0;
        ProtoCoef[1].a2 = 0;
        ProtoCoef[1].b0 = 1.0;
        ProtoCoef[1].b1 = 1.847759;
        ProtoCoef[1].b2 = 1.0;
				
       
		printf("%d",iir.length);

/*
 * Compute z-domain coefficients for each biquad section
 * for new Cutoff Frequency and Resonance
 */
       
	    for (nInd = 0; nInd < iir.length; nInd++)
        {
                 a0 = ProtoCoef[nInd].a0;
                 a1 = ProtoCoef[nInd].a1 / Q;
                 a2 = ProtoCoef[nInd].a2;

                 b0 = ProtoCoef[nInd].b0;
                 b1 = ProtoCoef[nInd].b1 / Q;      /* Divide by resonance or Q */
                 b2 = ProtoCoef[nInd].b2;
                 szxform(&a0, &a1, &a2, &b0, &b1, &b2, fc, fs, &k, coef);
                 coef += 4;                       /* Point to next filter section */
        }

        /* Update overall filter gain in coef array */
        iir.coef[0] = k;

        /* Display filter coefficients */
		printf("Lowpass filter coefficients:\n");
        for (nInd = 0; nInd < (iir.length * 4 + 1); nInd++){
                 printf("C[%d] = %15.10f\n", nInd, iir.coef[nInd]);
		}
   

return 0;
}



float iir_filter(float input,FILTER *iir){
    unsigned int i;
    float *hist1_ptr,*hist2_ptr,*coef_ptr;
    float output,new_hist,history1,history2;

/* allocate history array if different size than last call */

    if(!iir->history) {
        iir->history = (float *) calloc(2*iir->length,sizeof(float));
        if(!iir->history) {
            printf("\nUnable to allocate history array in iir_filter\n");
            exit(1);
        }
    }

    coef_ptr = iir->coef;                /* coefficient pointer */

    hist1_ptr = iir->history;            /* first history */
    hist2_ptr = hist1_ptr + 1;           /* next history */

        /* 1st number of coefficients array is overall input scale factor,
         * or filter gain */
    output = input * (*coef_ptr++);

    for (i = 0 ; i < iir->length; i++)
        {
        history1 = *hist1_ptr;           /* history values */
        history2 = *hist2_ptr;

        output = output - history1 * (*coef_ptr++);
        new_hist = output - history2 * (*coef_ptr++);    /* poles */

        output = new_hist + history1 * (*coef_ptr++);
        output = output + history2 * (*coef_ptr++);      /* zeros */

        *hist2_ptr++ = *hist1_ptr;
        *hist1_ptr++ = new_hist;
        hist1_ptr++;
        hist2_ptr++;
    }

    return(output);
}

/*
 * ----------------------------------------------------------
 *      Pre-warp the coefficients of a numerator or denominator.
 *      Note that a0 is assumed to be 1, so there is no wrapping
 *      of it.
 * ----------------------------------------------------------
 */
void prewarp(
    double *a0, double *a1, double *a2,
    double fc, double fs)
{
    double wp, pi;

    pi = 4.0 * atan(1.0);
    wp = 2.0 * fs * tan(pi * fc / fs);

    *a2 = (*a2) / (wp * wp);
    *a1 = (*a1) / wp;
}


/*
 * ----------------------------------------------------------
 * bilinear()
 *
 * Transform the numerator and denominator coefficients
 * of s-domain biquad section into corresponding
 * z-domain coefficients.
 *
 *      Store the 4 IIR coefficients in array pointed by coef
 *      in following order:
 *             beta1, beta2    (denominator)
 *             alpha1, alpha2  (numerator)
 *
 * Arguments:
 *             a0-a2   - s-domain numerator coefficients
 *             b0-b2   - s-domain denominator coefficients
 *             k               - filter gain factor. initially set to 1
 *                                and modified by each biquad section in such
 *                                a way, as to make it the coefficient by
 *                                which to multiply the overall filter gain
 *                                in order to achieve a desired overall filter
gain,
 *                                specified in initial value of k.
 *             fs             - sampling rate (Hz)
 *             coef    - array of z-domain coefficients to be filled in.
 *
 * Return:
 *             On return, set coef z-domain coefficients
 * ----------------------------------------------------------
 */
void bilinear(
    double a0, double a1, double a2,    /* numerator coefficients */
    double b0, double b1, double b2,    /* denominator coefficients */
    double *k,           /* overall gain factor */
    double fs,           /* sampling rate */
    float *coef         /* pointer to 4 iir coefficients */
)
{
    double ad, bd;

                 /* alpha (Numerator in s-domain) */
    ad = 4. * a2 * fs * fs + 2. * a1 * fs + a0;
                 /* beta (Denominator in s-domain) */
    bd = 4. * b2 * fs * fs + 2. * b1* fs + b0;

                 /* update gain constant for this section */
    *k *= ad/bd;

                 /* Denominator */
    *coef++ = (2. * b0 - 8. * b2 * fs * fs)
                           / bd;         /* beta1 */
    *coef++ = (4. * b2 * fs * fs - 2. * b1 * fs + b0)
                           / bd; /* beta2 */

                 /* Nominator */
    *coef++ = (2. * a0 - 8. * a2 * fs * fs)
                           / ad;         /* alpha1 */
    *coef = (4. * a2 * fs * fs - 2. * a1 * fs + a0)
                           / ad;   /* alpha2 */
}


/*
 * ----------------------------------------------------------
 * Transform from s to z domain using bilinear transform
 * with prewarp.
 *
 * Arguments:
 *      For argument description look at bilinear()
 *
 *      coef - pointer to array of floating point coefficients,
 *                     corresponding to output of bilinear transofrm
 *                     (z domain).
 *
 * Note: frequencies are in Hz.
 * ----------------------------------------------------------
 */
void szxform(
    double *a0, double *a1, double *a2, /* numerator coefficients */
    double *b0, double *b1, double *b2, /* denominator coefficients */
    double fc,         /* Filter cutoff frequency */
    double fs,         /* sampling rate */
    double *k,         /* overall gain factor */
    float *coef)         /* pointer to 4 iir coefficients */
{
                 /* Calculate a1 and a2 and overwrite the original values */
        prewarp(a0, a1, a2, fc, fs);
        prewarp(b0, b1, b2, fc, fs);
        bilinear(*a0, *a1, *a2, *b0, *b1, *b2, k, fs, coef);
}
